/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * d_printf.c: debugging messages. Can be easily compiled out by
 *   setting the -DNDEBUG flag. Verbosity level controlled by setting
 *   the environment variable DEBUG_LEVEL. If DEBUG_LEVEL = -1, then
 *   _all_ debugging messages will be printed. Otherwise, only those
 *   messages with a debug level less than DEBUG_LEVEL will be printed
 *   (so DEBUG_LEVEL = 0 produces no messages).
 *
 * $Id$
 */

#ifndef NDEBUG

#ifdef __KERNEL__
#include <linux/kernel.h>
#else
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif /* __KERNEL__ */

int debug_level = 0;
int debug_level_setp = 0;

void set_debug_level(void) {
#ifdef __KERNEL__
  debug_level = -1;
#else
  char *env_val;

  env_val = getenv("DEBUG_LEVEL");
  if (env_val != NULL) {
    debug_level = atoi(env_val);
  }
#endif /* __KERNEL__ */
  debug_level_setp = 1;
}

void _dprintf_func(int lvl, char *fmt, ...) {
  va_list ap;

  if (!debug_level_setp) {
    set_debug_level();
  }
  if (debug_level == -1 || lvl <= debug_level) {
    va_start(ap,fmt);
#ifdef __KERNEL__
    printk(fmt,ap);
#else
    vfprintf(stderr,fmt,ap);
#endif /* __KERNEL__ */
    va_end(ap);
  }
}
#endif /* !NDEBUG */
