/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * inject.c: The inject program may be used to submit a SNAP packet to
 *   a currently-running snapd process.
 *
 * $Id$
 */

#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "bytecode.h"
#include "d_printf.h"
#include "memalloc.h"
#include "io.h"

extern char *basename(const char *);

void parse_cmdline(int argc,char **argv);
int infd;
int udp_portnum = 7777;

int main(int argc,char **argv) {
  buffer_t inbuf;
  int sd;
  int packet_lenb;
  struct sockaddr_in snapd_addr;
  struct hostent *thishost;
  char hostname[128];
#ifndef NDEBUG
  char addrstr[16];
#endif /* !NDEBUG */

  parse_cmdline(argc,argv);
  
  packet_lenb = file_to_str(infd,&inbuf);
  if (packet_lenb < inbuf.lenb)
    inbuf.lenb = packet_lenb;

  if ((sd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
    perror("inject: socket");
    exit(1);
  }
  
  if (gethostname(hostname,sizeof(hostname)) < 0) {
    perror("inject: gethostname");
    exit(1);
  }
  if ((thishost = gethostbyname(hostname)) == NULL) {
    perror("inject: gethostbyname");
    exit(1);
  }

  snapd_addr.sin_family = AF_INET;
  snapd_addr.sin_port = udp_portnum;
  snapd_addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  /* memcpy(&(snapd_addr.sin_addr.s_addr),
     thishost->h_addr_list[0], thishost->h_length); */

  if (sendto(sd, inbuf.s, inbuf.lenb, 0,
	     (struct sockaddr *)&snapd_addr, sizeof(snapd_addr)) < 0) {
    perror("inject: sendto");
    exit(1);
  }

#ifndef NDEBUG
  sprintf_addr(addrstr,snapd_addr.sin_addr.s_addr);
  d_printf(10,"%s:%d: sent %d bytes to %s:%d\n",
	   __FILE__,__LINE__,inbuf.lenb,addrstr,udp_portnum);
#endif /* !NDEBUG */

  return(0);
}

void usage(int argc, char **argv) {
  printf("usage: %s [-?] [-p portnum] infile\n",basename(argv[0]));
  printf("\t-? : print this help\n");
  printf("\t-p : set UDP port (default=7777)\n");
}

void parse_cmdline(int argc, char **argv) {
  char *opts = "?p:";
  char c;
  int args_expected = 1;
  int args_processed = 0;
  int argidx;
  char *infilename;

  while((c = getopt(argc,argv,opts)) != EOF) {
    switch(c) {
    case '?':
      usage(argc,argv);
      exit(1);
    case 'p':
      udp_portnum = atoi(optarg);
      if (udp_portnum <= 0) {
	printf("%s: invalid port number %d\n",basename(argv[0]),
	       udp_portnum);
	usage(argc,argv);
	exit(1);
      }
    }
  }
  argidx = optind;
  while(args_processed < args_expected) {
    if (argidx >= argc) {
      /* missing arguments */
      printf("%s: missing argument(s)\n",basename(argv[0]));
      usage(argc,argv);
      exit(1);
    }
    switch(args_processed) {
    case 0:
      infilename = argv[argidx];
      if (strcmp(infilename,"-") == 0) {
	infd = 0;
      } else {
	infd = open(infilename,O_RDONLY);
	if (infd == -1) {
	  fprintf(stderr,"%s: unable to open file \"%s\" for input\n",
		  basename(argv[0]),infilename);
	  fflush(stderr);
	  exit(1);
	}
      }
      break;
    }
    argidx++;
    args_processed++;
  }
  if (argidx < argc) {
    /* extra arguments */
    printf("%s: extra argument(s)\n",basename(argv[0]));
    usage(argc,argv);
    exit(1);
  }
  return;
}

