/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * packet.h : definition of the SNAP packet format
 *
 * $Id$
 */

#ifdef __KERNEL__
#include <linux/skbuff.h>
#include <snap/bytecode.h>
#else
#include "bytecode.h"
#endif

#ifndef _SNAP_PACKET_H_
#define _SNAP_PACKET_H_

/* a heap is just a buffer with some meta-data off the front */
typedef struct {
  int lenb;			/* for now, length in bytes */
  char *h;			/* the heap */
} heap_t;

/* packet header layout */
#ifdef __KERNEL__
#include <net/snap.h>
typedef struct snaphdr header_t;
#else
/* all of the fields in the header are in network byte order
   (that is, they are little-endian */
typedef struct {
  unsigned int src;		/* source address */
  unsigned int dst;		/* destination address */
  unsigned short sport;		/* source port */
  unsigned short rb;		/* resource bound */
  unsigned short entry_point;	/* which instruction to start with */
  unsigned short code_sizeb;	/* code length, in bytes */
  unsigned short heap_sizeb;	/* length of heap, in bytes */
  unsigned short stack_sizeb;	/* length of stack, in bytes */
  unsigned int is_little_endian;/* endianness indicator */
  char rest[0];			/* code, heap, stack */
} header_t;
#endif /* __KERNEL__ */

/* active packets */
typedef struct {
  header_t *hdr;
  /* code */
  instr_t *code_min;
  instr_t *pc;
  instr_t *code_max;
  /* stack */
  value_t *stack_min;
  value_t *sp; 		  /* points above top value (i.e. alloc, then incr) */
  value_t *stack_max;
  /* in-packet heap */
  void *heap_min;
  void *h_alloc_ptr; 	  /* used only during assembly */
  void *heap_max;
  /* heap allocation pointer at runtime is stack_max, growing down;
     value pointed to by stack_max is legal (i.e. decr, then alloc) */
  void *h_alloc_heap_max; /* essentially the top of the buffer */
  /* used for optimizing marshalling---allows the marshaller to
     assume that the packet is already marshalled, starting at the
     hdr address. */
  unsigned int is_contiguous;
#ifdef __KERNEL__
  struct sk_buff *skb;
#endif /* __KERNEL__ */
} packet_t;

#ifndef __KERNEL__
extern void fprintf_packet(FILE *f,packet_t *p);
#endif /* !__KERNEL__ */

#endif /* _SNAP_PACKET_H_ */
