/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * router.c : reads the Linux routing table from /proc/net/route and
 *   implements a simple-minded routing lookup engine
 *
 * $Id$
 */

#include <assert.h>
#include <net/route.h>
#include <netinet/in.h>
#include "bytecode.h"
#include "d_printf.h"
#include "memalloc.h"
#include "router.h"
#include "proc.h"

struct rtentry *routes = NULL;
int num_entries = 0;
int num_routes = 0;

void read_routes(void) {
  char buf[1024];
  FILE *f;
  int nr;
  struct rtentry *curr_rt;
  char *fmt;
#ifndef NDEBUG
  char addrstr1[16],addrstr2[16],addrstr3[16];
#endif /* !NDEBUG */

  char iface[16];
  unsigned int dest_addr, gw_addr, mask;
  unsigned int flags,refcnt,use,metric,mtu,window,irtt;
  
  f = fopen(PROC_NET_ROUTE_PATH,"r");
  if (f == NULL) {
    fprintf(stderr,"%s:%d: unable to open %s\n",
	    __FILE__,__LINE__,PROC_NET_ROUTE_PATH);
    exit(1);
  }

  nr = 0;
  /* count number of routes */
  while(fgets(buf,sizeof(buf),f)) {
    nr++;
  }
  nr--;				/* strip off title line */
  d_printf(10,"%s:%d: reading %d routes\n",__FILE__,__LINE__,nr);
	    
  /* do we have enough room in the routing table? */
  if (nr > num_entries) {
    if (routes != NULL) {
      free(routes);
    }
    memalloc(routes,struct rtentry *,sizeof(struct rtentry) * nr);
    num_entries = nr;
  }

  /* now read in the routes */
  rewind(f);
  fmt = proc_gen_fmt(PROC_NET_ROUTE_PATH, f,
		     "Iface", "%16s",
		     "Destination", "%X",
		     "Gateway", "%X",
		     "Flags", "%X",
		     "RefCnt", "%d",
		     "Use", "%d",
		     "Metric", "%d",
		     "Mask", "%X",
		     "MTU", "%d",
		     "Window", "%d",
		     "IRTT", "%d",
		     NULL);
  if (fmt == NULL) {
    fprintf(stderr,"%s:%d: did not get format for reading %s\n",__FILE__,
	    __LINE__,PROC_NET_ROUTE_PATH);
    exit(1);
  }

  curr_rt = routes;
  while(fgets(buf,sizeof(buf)-1,f)) {
    assert(curr_rt >= &routes[0]);
    assert(curr_rt < &routes[nr]); 
    if (sscanf(buf,fmt,
	       iface,&dest_addr,&gw_addr,&flags,&refcnt,&use,
	       &metric,&mask,&mtu,&window,&irtt) < 11)
      continue;

#ifndef NDEBUG    
    sprintf_addr(addrstr1,dest_addr);
    sprintf_addr(addrstr2,gw_addr);
    sprintf_addr(addrstr3,mask);
    d_printf(10,"%s:%d: route : dst = %s , gw = %s , mask = %s\n",
	    __FILE__,__LINE__,addrstr1,addrstr2,addrstr3);
#endif /* !NDEBUG */

    curr_rt->rt_dst.sa_family = AF_INET;
    memcpy(&(curr_rt->rt_dst.sa_data),&dest_addr,sizeof(unsigned int));
    curr_rt->rt_gateway.sa_family = AF_INET;
    memcpy(&(curr_rt->rt_gateway.sa_data),&gw_addr,sizeof(unsigned int));
    curr_rt->rt_genmask.sa_family = AF_INET;
    memcpy(&(curr_rt->rt_genmask.sa_data),&mask,sizeof(unsigned int));
    curr_rt->rt_flags = (unsigned short int)flags;
    curr_rt->rt_metric = (short int)metric;
    curr_rt->rt_mtu = (unsigned long int)mtu;
    curr_rt->rt_window = (unsigned long int)window;
    curr_rt->rt_irtt = (unsigned short int)irtt;

    curr_rt++;
  }
  num_routes = curr_rt - routes;
}

unsigned int *nexthop(unsigned int dst) {
  int i;

  print_anti_timer(16,"nexthop");
  for(i=0; i<num_routes; i++) {
    struct rtentry *rt = &routes[i];
    unsigned int *dest_net = (unsigned int *)&(rt->rt_dst.sa_data);
    unsigned int *gw = (unsigned int *)&(rt->rt_gateway.sa_data);
    unsigned int *mask = (unsigned int *)&(rt->rt_genmask.sa_data);
    if ((dst & *mask) == *dest_net) {
      print_timer(16,"nexthop");
      return(gw);
    }
  }
  print_timer(16,"nexthop");
  return(NULL);
}
