/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * snapd.c : SNAP user-space node daemon. Receives packets on a
 *   dedicated UDP port and processes them, sending new packets out
 *   over UDP as necessary.
 *
 * $Id$
 */

#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "interp.h"
#include "packet.h"
#include "router.h"
#include "timers.h"
#include "udpnet.h"
#include "services.h"
#include "memalloc.h"

extern char *basename(const char *);

#define SVC_BUF_SZ 5012

void finish(int ignore);
void parse_cmdline(int argc, char **argv);

int main(int argc, char **argv) {
  packet_t *p;
#ifdef SERVICES
  buffer_t service_heap;
#endif

  parse_cmdline(argc,argv);

  read_routes();
  init_udpnet();
  signal(SIGINT,finish);

#ifdef SERVICES
  /* set up heap for storing service values */
  memalloc(service_heap.s,char *,SVC_BUF_SZ);
  service_heap.lenb = SVC_BUF_SZ;
  init_services(&service_heap);
#endif

#ifndef TIMERS_OFF
  init_all_timers();
  print_anti_timer(0,"total");
#endif /* !TIMERS_OFF */

  p=udp_get_packet();
  if (p != NULL) 
    interp_packet(p);
  while(1) {
    print_anti_timer(2,"daemon_loop");
    p = udp_get_packet();
    if (p != NULL) 
      interp_packet(p);
    print_timer(2,"daemon_loop");
  }
  return(0);
}

void finish(int ignore) {
  close_udpnet();
#ifndef TIMERS_OFF
  print_timer(0,"total");
  dump_all_timers();
#endif /* !TIMERS_OFF */
  exit(0);
}

void usage(int argc, char **argv) {
  printf("usage: %s [-?i] [-p portnum] [-t timerflags] [-l localaddr]\n",
	 basename(argv[0]));
  printf("\t-? : print this help\n");
  printf("\t-p : set UDP port (default=7777)\n");
  printf("\t-t : which timers to print\n");
  printf("\t-i : print individual timers\n");
  printf("\t-l : set local address (default=`hostname`)\n");
}

void parse_cmdline(int argc, char **argv) {
  char *opts = "?p:t:il:";
  char c;

  do_print_individual_timers = 0;
  while((c = getopt(argc,argv,opts)) != EOF) {
    switch(c) {
    case '?':
      usage(argc,argv);
      exit(1);
    case 'p':
      udp_portnum = atoi(optarg);
      break;
    case 't':
      print_flags = optarg;
      print_flag_count = strlen(optarg);
      break;
    case 'i':
      do_print_individual_timers = 1;
      break;
    case 'l': {
      unsigned int b1,b2,b3,b4;
      unsigned int haddr;
      if (sscanf(optarg,"%d.%d.%d.%d",&b1,&b2,&b3,&b4) == 4) {
	haddr = (b1 << 24) | (b2 << 16) | (b3 << 8) | b4;
	thisaddr.s_addr = htonl(haddr);
      }
      break;
    }
    }
  }
  if (optind != argc) {
    printf("%s: unexpected argument(s)\n",basename(argv[0]));
    usage(argc,argv);
    exit(1);
  }
}
