/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore and Michael Hicks.
 *
 * snaplex.l : lexer for SNAP assembly language
 *
 * $Id$
 */

%{
#include <assert.h>
#include <ctype.h>
#include <netinet/in.h>
#include <string.h>
#include "config.h"
#include "io.h"
#include "memalloc.h"
#include "snapparse.tab.h"

int value_int;
uint32 value_addr;
buffer_t value_str;
int value_exc;
float32 value_float;

void conv_string(char *s,buffer_t *buf);

%}

ws	[ \t\n]
digit	[0-9]
addr	{digit}+\.{digit}+\.{digit}+\.{digit}+
strchar [^\\\"\n]|\\\\|\\\"|\\n|\\b|\\r|\\t|\\{digit}{digit}{digit}
A       [aA]
B       [bB]
C       [cC]
D       [dD]
E       [eE]
F       [fF]
G       [gG]
H       [hH]
I       [iI]
J       [jJ]
K       [kK]
L       [lL]
M       [mM]
N       [nN]
O       [oO]
P       [pP]
Q       [qQ]
R       [rR]
S       [sS]
T       [tT]
U       [uU]
V       [vV]
W       [wW]
X       [xX]
Y       [yY]
Z       [zZ]

%%
{ws}+		;	/* skip whitespace */
\;.*\n		;	/* ignore comments to end of line */

{E}{X}{I}{T}	return(T_EXIT);
{P}{U}{S}{H}	return(T_PUSH);
{P}{O}{P}{I}	return(T_POPI);
{P}{O}{P}	return(T_POP);
{P}{U}{L}{L}	return(T_PULL);
{S}{T}{O}{R}{E}	return(T_STORE);
{E}{Q}{I}	return(T_EQI);
{E}{Q}		return(T_EQ);
{N}{E}{Q}{I}	return(T_NEQI);
{N}{E}{Q}	return(T_NEQ);
{J}{I}		return(T_JI);
{P}{A}{J}	return(T_PAJ);
{T}{P}{A}{J}	return(T_TPAJ);
{B}{E}{Z}	return(T_BEZ);
{B}{N}{E}	return(T_BNE);
{A}{D}{D}{I}	return(T_ADDI);
{A}{D}{D}	return(T_ADD);
{S}{U}{B}{I}	return(T_SUBI);
{S}{U}{B}	return(T_SUB);
{M}{U}{L}{T}{I}	return(T_MULTI);
{M}{U}{L}{T}	return(T_MULT);
{D}{I}{V}{I}	return(T_DIVI);
{D}{I}{V}	return(T_DIV);
{M}{O}{D}{I}	return(T_MODI);
{M}{O}{D}	return(T_MOD);
{N}{E}{G}	return(T_NEG);
{N}{O}{T}	return(T_NOT);
{L}{N}{O}{T}	return(T_LNOT);
{A}{N}{D}{I}	return(T_ANDI);
{A}{N}{D}	return(T_AND);
{O}{R}{I}	return(T_ORI);
{O}{R}		return(T_OR);
{L}{S}{H}{L}{I}	return(T_LSHLI);
{L}{S}{H}{L}	return(T_LSHL);
{R}{S}{H}{L}{I}	return(T_RSHLI);
{R}{S}{H}{L}	return(T_RSHL);
{R}{S}{H}{A}{I}	return(T_RSHAI);
{R}{S}{H}{A}	return(T_RSHA);
{S}{N}{E}{T}{I}	return(T_SNETI);
{S}{N}{E}{T}	return(T_SNET);
{B}{C}{A}{S}{T}{I}	return(T_BCASTI);
{B}{C}{A}{S}{T}		return(T_BCAST);
{I}{S}{X}		return(T_ISX);
{G}{E}{T}{R}{B}		return(T_GETRB);
{G}{E}{T}{S}{R}{C}	return(T_GETSRC);
{G}{E}{T}{D}{S}{T}	return(T_GETDST);
{G}{E}{T}{S}{P}{T}	return(T_GETSPT);
{H}{E}{R}{E}		return(T_HERE);
{I}{S}{H}{E}{R}{E}	return(T_ISHERE);
{R}{O}{U}{T}{E}		return(T_ROUTE);
{S}{E}{N}{D}		return(T_SEND);
{H}{O}{P}		return(T_HOP);
{F}{O}{R}{W}		return(T_FORW);
{F}{O}{R}{W}{T}{O}	return(T_FORWTO);
{D}{E}{M}{U}{X}{I}	return(T_DEMUXI);
{D}{E}{M}{U}{X}		return(T_DEMUX);
{P}{R}{I}{N}{T}		return(T_PRINT);
{P}{C}			return(T_PC);
{M}{K}{T}{U}{P}		return(T_MKTUP);
{N}{T}{H}		return(T_NTH);
{L}{E}{N}		return(T_LEN);
{I}{S}{T}{U}{P}		return(T_ISTUP);
{S}{E}{N}{D}{C}{K}	return(T_SENDCK);
{H}{O}{P}{C}{K}		return(T_HOPCK);
{S}{V}{C}{V}		return(T_SVCV);
{C}{A}{L}{L}{S}		return(T_CALLS);
{G}{E}{Q}   		return(T_GEQ);
{G}{T}   		return(T_GT);
{L}{E}{Q}   		return(T_LEQ);
{L}{T}   		return(T_LT);
{G}{E}{Q}{I}		return(T_GEQI);
{G}{T}{I}		return(T_GTI);
{L}{E}{Q}{I}		return(T_LEQI);
{L}{T}{I}		return(T_LTI);
#{S}{R}{C}		return(T_SRC);
#{D}{S}{T}		return(T_DST);
#{R}{B}			return(T_RB);
#{S}{P}{O}{R}{T}	return(T_SPORT);
#{D}{A}{T}{A}		return(T_DATA);

main:			return(T_MAIN);
[+]			return(T_PLUS);
[-]			return(T_MINUS);
[(]			return(T_LPAREN);
[)]			return(T_RPAREN);
[,]			return(T_COMMA);

{addr}		{ char *s = yytext;
		  char *s1;
		  char *s2;
		  char *s3;
		  char *s4;
                  uint32 value_addr_h;
		  s1 = strsep(&s,".");
		  value_addr_h = (uint32)(atoi(s1) << 24);
		  s2 = strsep(&s,".");
		  value_addr_h |= (uint32)(atoi(s2) << 16);
		  s3 = strsep(&s,".");
		  value_addr_h |= (uint32)(atoi(s3) << 8);
		  s4 = s;
		  value_addr_h |= (uint32)(atoi(s4));
		  value_addr = htonl(value_addr_h);
		  return(T_ADDRV);
		}
\|{digit}+\|	{ int sizeb;
		  sscanf(yytext,"|%d|",&sizeb);
		  memalloc(value_str.s,char *,sizeb);
		  value_str.lenb = sizeb;
		  value_str.s[sizeb-1] = (char)0xff;
		  return(T_STRV);
		}
[-]?{digit}+	{ value_int = atoi(yytext);
		  return(T_INTV);
		}
[-]?{digit}*[.]{digit}+ 	{ int ntok;
		  ntok = sscanf(yytext,"%f",&value_float);
		  if (ntok != 1) {
		    fprintf(stderr,
			    "%s:%d: failure to parse float literal %s\n",
			    __FILE__,__LINE__,yytext);
		    exit(1);
		  }
		  return(T_FLOATV);
		}
{digit}+[U]	{ int len = strlen(yytext);
		  char c = yytext[len-1];
		  yytext[len-1] = '\0';
		  value_addr = atoi(yytext);
		  yytext[len-1] = c;
		  return(T_ADDRV);
		}
\"{strchar}*\"  { conv_string(yytext,&value_str);
		  return(T_STRV);
                }
[E][_]{digit}+  { value_exc = atoi(&yytext[2]);
		  return(T_EXCV); 
		}
[a-z][[:alnum:]_]+ { memalloc(value_str.s, char *, strlen(yytext)+1); 
		  strcpy(value_str.s,yytext);
		  value_str.lenb = strlen(yytext) + 1;
		  return(T_LABELV);
		}
[a-z][[:alnum:]_]+[:] { memalloc(value_str.s, char *, strlen(yytext)); 
		  strncpy(value_str.s,yytext,strlen(yytext)-1);
		  value_str.lenb = strlen(yytext);
		  return(T_LABEL); }

%%
/* <<EOF>>		return(T_EOF); */
void conv_string(char *orig,buffer_t *buf) {
  char *dest;
  char *src;

  memalloc(buf->s, char *, strlen(orig) - 1);
  dest = buf->s;
  src = orig;		
  src++;                        /* skip initial quote */
  while(*src != '\"') {
    if (*src == '\\') {
      switch(src[1]) {
      case 'n':
	*dest++ = '\n'; src += 2; break;
      case 'b':
	*dest++ = '\b'; src += 2; break;
      case 'r':
	*dest++ = '\r'; src += 2; break;
      case 't':
	*dest++ = '\t'; src += 2; break;
      case '\\':
	*dest++ = '\\'; src += 2; break;
      case '\"':
	*dest++ = '\"'; src += 2; break;
      default: 
	{
	  char save = src[4];
	  int charcode;
	  assert((isdigit(src[1]) && isdigit(src[2]) && isdigit(src[3])));
	  src[4] = '\0';
	  sscanf(&src[1],"%o",&charcode);
	  src[4] = save;
	  *dest++ = (char)charcode; src += 4; break;
	}
      }
    } else {
      *dest++ = *src++;
    }
  }
  *dest++ = '\0';
  buf->lenb = dest - buf->s;
  return;
}
