/* snap-1.0. Copyright (C) 2000 by Jonathan T. Moore, Michael Hicks,
 *                                 and Scott Nettles.
 *
 * timers.h : low-overhead timers based on the Pentium cycle counter
 *   for flexible performance profiling.
 *
 * $Id$
 */

#ifndef _SNAP_TIMERS_H
#define _SNAP_TIMERS_H

/*

If you want to time a .c file (all that's supported right now), you
first add a new timer to the file timers.c.
Right now there are two timers defined:

struct timert_t timerlist[] =
{
ENTRY(0,        "total_time"),
ENTRY(1,        "timer_time"),
};

To add a new one, add it to the list:

struct timert_t timerlist[] =
{
ENTRY(0,        "total_time"),
ENTRY(1,        "timer_time"),
ENTRY(2,        "my_new_timer"),
};

Then, in you source file, you include the file "timers.h" and
make use of the macros print_anti_timer to start a timer
and print_timer to stop it:

  print_anti_timer(0,"total_time");
  sleep(2);
  print_timer(0,"total_time");

To make use of the timing system, the global variables
print_flags and do_print_individual_timers must be set properly.
If do_print_individual_timers is set to 1, then each time
the time is taken, a message will be printed.  This is useful
for seeing the ordering of events, but causes more timing overhead.
print_flags indicates which timers you want to turn on for your
run.  It is a char* where each element of the array is '1' or '0'.
If the value of the xth entry is '1' then all print_timer and
print_anti_timer calls for timer x will be enabled.

To see the results of the timings, the call dump_all_timers() is
used.  It prints the average elapsed time and total elapsed time
for each timer.

The underlying mechanism of the timers is the pentium cycle counter.
When you build, a program will be run to determine the clockrate
on your machine so that cycles may be properly converted to microsecs.

*/

extern char *print_flags;
extern int print_flag_count;
extern int do_print_individual_timers;
extern int do_print_item_messages;
extern int do_print_antitimers;

extern void init_all_timers(void);
extern void dump_all_timers(void);

/*
#define TIMERS_OFF
*/
#ifdef __KERNEL__
#define TIMERS_OFF
#endif /* __KERNEL__ */


#ifndef TIMERS_OFF

extern void internal_print_time(int index, char *label);
extern void internal_print_anti_time(int index, char *label);

#define print_timer(index, string)  \
  if ((print_flags[index] == '1') && (index < print_flag_count)) {  \
    internal_print_time(index, string);  \
  }

#define print_anti_timer(index, string)  \
  if ((print_flags[index] == '1') && (index < print_flag_count)) {  \
    internal_print_anti_time(index, string);  \
  }

#ifndef MINOR_TIMERS_OFF

#define print_mtimer(index, string)  \
  if ((print_flags[index] == '1') && (index < print_flag_count)) {  \
    internal_print_time(index, string);  \
  }

#define print_anti_mtimer(index, string)  \
  if ((print_flags[index] == '1') && (index < print_flag_count)) {  \
    internal_print_anti_time(index, string);  \
  }
#else
#define print_mtimer(x, y) {}
#define print_anti_mtimer(x, y) {}
#endif
#else
#define print_timer(x, y) {}
#define print_anti_timer(x, y) {}
#define print_mtimer(x, y) {}
#define print_anti_mtimer(x, y) {}
#endif

#endif /* _SNAP_TIMERS_H */

